/*
 * JBoss 5 POJO service
 * Copyright (c) 2009 samaxes.com
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.samaxes.jboss.service.plain;

import java.net.URL;

import org.jboss.dependency.spi.ControllerContext;
import org.jboss.kernel.Kernel;
import org.jboss.kernel.spi.dependency.KernelController;
import org.testng.annotations.AfterSuite;
import org.testng.annotations.BeforeSuite;

/**
 * Base test class.
 */
public class BaseTest<T> {

    private URL url;

    private EmbeddedBootstrap bootstrap;

    private Kernel kernel;

    private KernelController controller;

    /**
     * Starts up the Embedded Jboss and sets the data sources.
     * 
     * @throws Exception
     */
    @BeforeSuite
    public void setUpSuite() throws Exception {
        // Start JBoss Microcontainer
        bootstrap = new EmbeddedBootstrap();
        bootstrap.run();

        kernel = bootstrap.getKernel();
        controller = kernel.getController();
    }

    /**
     * Shuts down the Embedded Jboss.
     */
    @AfterSuite
    public void tearDownSuite() {
    }

    /**
     * Deploys a deployment descriptor.
     * 
     * @param url the deployment descriptor filename
     */
    protected void deploy(String url) {
        ClassLoader cl = Thread.currentThread().getContextClassLoader();
        this.url = cl.getResource(url);

        bootstrap.deploy(cl.getResource(url));
    }

    /**
     * Undeploys a deployment descriptor.
     */
    protected void undeploy() {
        bootstrap.undeploy(url);
    }

    /**
     * Gets a service by its context name.
     * 
     * @param name the service name
     * @return the service
     */
    @SuppressWarnings( { "unchecked" })
    protected T getService(String name) {
        ControllerContext context = controller.getInstalledContext(name);

        return (context == null) ? null : (T) context.getTarget();
    }
}
